import { FC, useEffect, useState } from "react";

import { useConnection, useWallet } from "@solana/wallet-adapter-react";
import {
  SOLCIAL_SWAP,
  SOLCIAL_MINT,
  getUserMint,
  getUserSwapAccount,
  getUsers,
  loadTokenSwapAccount,
  getTokenPoolBalance,
  displayPrice,
  getTokenMetadata,
} from "@solcial.io/solana-client";
import { Connection, PublicKey } from "@solana/web3.js";
import {
  ASSOCIATED_TOKEN_PROGRAM_ID,
  Token,
  TOKEN_PROGRAM_ID,
} from "@solana/spl-token";

async function getUserBallance(connection: Connection, swap, user: PublicKey) {
  const tokenAUserAccount = await Token.getAssociatedTokenAddress(
    ASSOCIATED_TOKEN_PROGRAM_ID,
    TOKEN_PROGRAM_ID,
    swap.mintA,
    user
  );
  const tokenBUserAccount = await Token.getAssociatedTokenAddress(
    ASSOCIATED_TOKEN_PROGRAM_ID,
    TOKEN_PROGRAM_ID,
    swap.mintB,
    user
  );

  let tokenA;
  try {
    tokenA = await connection.getTokenAccountBalance(tokenAUserAccount);
  } catch {}
  let tokenB;
  try {
    tokenB = await connection.getTokenAccountBalance(tokenBUserAccount);
  } catch {}

  return {
    tokenA,
    tokenB,
  };
}

export const SwapView: FC<{ swapId: string }> = ({ swapId }) => {
  const { connection } = useConnection();
  const wallet = useWallet();
  const [swap, setSwap] = useState(null);
  const [poolBalance, setPoolBalance] = useState(null);
  const [tokenMeta, setTokenMeta] = useState(null);
  const [tokenJsonMeta, setTokenJsonMeta] = useState(null);
  const [userBallance, setUserBallance] = useState(null);

  useEffect(() => {
    async function load() {
      if (swapId != null) {
        const accountId = new PublicKey(swapId);
        const s = await loadTokenSwapAccount(connection, accountId);
        setSwap(s);
      }
    }
    load();
  }, [swapId]);

  useEffect(() => {
    async function load() {
      if (swap != null) {
        const b = await getTokenPoolBalance(connection, swap);
        const meta = await getTokenMetadata(connection, swap);
        const user = await getUserBallance(connection, swap, wallet.publicKey);

        setPoolBalance(b);
        setTokenMeta(meta);
        setUserBallance(user);
      }
    }
    load();
  }, [swap]);

  useEffect(() => {
    async function load() {
      if (tokenMeta != null) {
        console.log(tokenMeta);

        if (tokenMeta.tokenA && tokenMeta.tokenA.uri) {
          console.log("Fetching ", tokenMeta.tokenA.uri);
          const metaJsonA = await fetch(tokenMeta.tokenA.uri);
          if (metaJsonA.ok) {
            const bodyA = await metaJsonA.json();
            console.log(bodyA);
            setTokenJsonMeta(bodyA);
          }
        }

        if (tokenMeta.tokenB && tokenMeta.tokenB.uri) {
          console.log("Fetching ", tokenMeta.tokenB.uri);
          const metaJsonB = await fetch(tokenMeta.tokenB.uri);
          if (metaJsonB.ok) {
            const bodyB = await metaJsonB.json();
            console.log(bodyB);
            setTokenJsonMeta(bodyB);
          }
        }
      }
    }
    load();
  }, [tokenMeta]);

  let tokenBTicker = null;
  if (swap && swap.tokenB === SOLCIAL_MINT) {
    tokenBTicker = "SLCL";
  }

  return (
    <div className="md mx-auto p-4 flex flex-col">
      <div className="stats bg-primary text-primary-content mx-3">

        <div className="stat">
          <div className="stat-title">Token A</div>
          <div className="stat-desc">
            {tokenMeta &&  tokenMeta.tokenA ? tokenMeta.tokenA.name : ""}
          </div>
          <div className="stat-desc">
            {tokenMeta &&  tokenMeta.tokenA ? tokenMeta.tokenA.symbol : ""}
          </div>
          <div className="stat-desc">
            {tokenMeta &&  tokenMeta.tokenA? tokenMeta.tokenA.uri : ""}
          </div>
          <div className="stat-value">
            {poolBalance && displayPrice(poolBalance.tokenA) || 0}
          </div>
          <div className="stat-actions">
            <button className="btn btn-sm btn-success">Add funds</button>
          </div>
        </div>

        <div className="stat">
          <div className="stat-title">
            Token B
          </div>
          <div className="stat-value">
            {tokenBTicker} {poolBalance && displayPrice(poolBalance.tokenB) || 0}
          </div>
          <div className="stat-actions">
            <button className="btn btn-sm btn-success">Add funds</button>
          </div>
        </div>
      </div>

<div className="flex flex-row items-start justify-around">
      <div className="card w-auto bg-neutral text-neutral-content">
        <div className="card-body items-center text-center">
          <h2 className="card-title">Solcial swap </h2>
          <pre className="code">
            {JSON.stringify(toObject(swap),null,2)}
          </pre>
          </div>
          </div>
      <div className="card w-96 bg-neutral text-neutral-content">
        <div className="card-body items-center text-center">
          <h2 className="card-title">Solcial swap </h2>

          <div className="form-control">
            <label className="input-group">
              <input
                type="text"
                placeholder="0.01"
                className="input input-bordered"
              />
              <span>
                <div className="avatar placeholder">
                  <div className="bg-slate-500 text-neutral-content rounded-full w-8">
                    <span className="text-xs">AA</span>
                  </div>
                </div>
                BTC
              </span>
            </label>
            <label className="label">
              <span className="label-text-alt">Balance : 0</span>
            </label>
          </div>

          <div className="form-control p-1">
            <button type="button" className="btn btn-circle btn-outline btn-sm">
              <svg
                viewBox="0 0 24 25"
                fill="none"
                xmlns="http://www.w3.org/2000/svg"
                width="24"
                height="24"
              >
                <path
                  d="M8.1 1.588L9.619 3.95h-.225c-3.3 0-5.682 2.363-5.682 5.625 0 3.263 2.382 5.625 5.682 5.625v-1.875c-2.27 0-3.807-1.5-3.807-3.75 0-2.231 1.52-3.75 3.807-3.75h.3L8.119 8.263l1.575 1.012 2.812-4.35L9.694.575 8.1 1.588zm10.331 13.837c0 2.231-1.519 3.75-3.806 3.75h-.3l1.575-2.438-1.575-1.012-2.813 4.35 2.813 4.35 1.575-1.013-1.519-2.362h.225c3.3 0 5.681-2.363 5.681-5.625 0-3.263-2.381-5.625-5.681-5.625v1.875c2.287.019 3.825 1.519 3.825 3.75z"
                  fill="currentColor"
                ></path>
              </svg>
            </button>
          </div>

          <div className="form-control">
            <label className="input-group">
              <input
                type="text"
                placeholder="0.01"
                className="input input-bordered"
              />
              <span>
                <div className="avatar">
                  <div className="bg-slate-500 text-neutral-content rounded-full w-8">
                    <img src="https://placeimg.com/192/192/people" />
                  </div>
                </div>
                BTC
              </span>
            </label>
            <label className="label">
              <span className="label-text-alt">Balance : 0</span>
            </label>
          </div>

          <div className="card-actions justify-end">
            <button className="btn btn-primary">Swap</button>
          </div>
        </div>
        </div>
      </div>
    </div>
  );
};


function toObject(swap) {
  return JSON.parse(JSON.stringify(swap, (key, value) =>
      typeof value === 'bigint'
          ? value.toString()
          : value // return everything else unchanged
  ));
}